import pytest

from mm_aptos import ans_async

pytestmark = pytest.mark.anyio


async def test_address_to_primary_name():
    address = "0x68e6982c788b50e3caccc834a4764763381d7201be996914e1310139a35854ed"
    assert (await ans_async.address_to_primary_name(address)).unwrap() == "vitalik"

    address = "0xabfabdec0732564bd906fb94e467410a131c6e6040f7bca860458e2026e3b14e"
    assert (await ans_async.address_to_primary_name(address)).unwrap() is None



import pytest
from mm_std import get_dotenv


@pytest.fixture
def anyio_backend() -> str:
    return "asyncio"


@pytest.fixture
def mainnet_rpc_url() -> str:
    return get_dotenv("MAINNET_RPC_URL")


@pytest.fixture
def okx_address() -> str:
    return "0x834d639b10d20dcb894728aa4b9b572b2ea2d97073b10eacb111f338b20ea5d7"



import pytest

from mm_aptos import balance_async
from mm_aptos.coin import APTOS_COIN_TYPE

pytestmark = pytest.mark.anyio


async def test_get_balance_async(mainnet_rpc_url, okx_address):
    res = await balance_async.get_balance(mainnet_rpc_url, okx_address, APTOS_COIN_TYPE)
    assert res.unwrap() > 1000



from mm_std import DataResult, http_request


async def get_balance(
    node: str, account_address: str, coin_type: str, timeout: float = 5.0, proxy: str | None = None
) -> DataResult[int]:
    url = f"{node}/accounts/{account_address}/resource/0x1::coin::CoinStore%3C{coin_type}%3E"
    res = await http_request(url, proxy=proxy, timeout=timeout)
    try:
        json_res = res.parse_json_body()
        if json_res.get("error_code") == "resource_not_found":
            return DataResult(ok=0, data=res.model_dump())
        return DataResult(ok=int(json_res["data"]["coin"]["value"]), data=res.model_dump())
    except Exception as err:
        return DataResult(err="exception", data={"response": res.model_dump(), "exception": str(err)})



from decimal import Decimal

from mm_crypto_utils import Nodes, Proxies, random_node, random_proxy
from mm_std import Err, Ok, Result, hr, hra


def get_balance(nodes: str, account_address: str, coin_type: str, timeout: float = 5.0, proxy: str | None = None) -> Result[int]:
    url = f"{random_node(nodes)}/accounts/{account_address}/resource/0x1::coin::CoinStore%3C{coin_type}%3E"
    res = hr(url, proxy=proxy, timeout=timeout)
    try:
        if res.json.get("error_code") == "resource_not_found":
            return Ok(0, data=res.to_dict())
        return Ok(int(res.json["data"]["coin"]["value"]), data=res.to_dict())
    except Exception as err:
        return Err(err, data=res.to_dict())


async def get_balance_async(
    nodes: str, account_address: str, coin_type: str, timeout: float = 5.0, proxy: str | None = None
) -> Result[int]:
    url = f"{random_node(nodes)}/accounts/{account_address}/resource/0x1::coin::CoinStore%3C{coin_type}%3E"
    res = await hra(url, proxy=proxy, timeout=timeout)
    try:
        if res.json.get("error_code") == "resource_not_found":
            return Ok(0, data=res.to_dict())
        return Ok(int(res.json["data"]["coin"]["value"]), data=res.to_dict())
    except Exception as err:
        return Err(err, data=res.to_dict())


def get_balance_with_retries(
    retries: int, nodes: Nodes, account_address: str, coin_type: str, timeout: float = 5.0, proxies: Proxies = None
) -> Result[int]:
    res: Result[int] = Err("not started yet")
    for _ in range(retries):
        res = get_balance(random_node(nodes), account_address, coin_type, timeout=timeout, proxy=random_proxy(proxies))
        if res.is_ok():
            return res
    return res


async def get_balance_async_with_retries(
    retries: int, nodes: Nodes, account_address: str, coin_type: str, timeout: float = 5.0, proxies: Proxies = None
) -> Result[int]:
    res: Result[int] = Err("not started yet")
    for _ in range(retries):
        res = await get_balance_async(
            random_node(nodes), account_address, coin_type, timeout=timeout, proxy=random_proxy(proxies)
        )
        if res.is_ok():
            return res
    return res


def get_decimal_balance_with_retries(
    retries: int,
    nodes: Nodes,
    account_address: str,
    coin_type: str,
    decimals: int,
    round_ndigits: int = 5,
    timeout: float = 5.0,
    proxies: Proxies = None,
) -> Result[Decimal]:
    return get_balance_with_retries(retries, nodes, account_address, coin_type, timeout=timeout, proxies=proxies).and_then(
        lambda o: Ok(round(Decimal(o / 10**decimals), ndigits=round_ndigits))
    )



APTOS_COIN_TYPE = "0x1::aptos_coin::AptosCoin"
APTOS_COIN_DECIMALS = 8

LAYERZERO_USD_COIN_TYPE = "0xf22bede237a07e121b56d91a491eb7bcdfd1f5907926a9e58338f964a01b17fa::asset::USDC"
LAYERZERO_USD_COIN_DECIMALS = 6

DITTO_STAKED_APTOS_COIN_TYPE = "0xd11107bdf0d6d7040c6c0bfbdecb6545191fdf13e8d8d259952f53e1713f61b5::staked_coin::StakedAptos"
DITTO_STAKED_APTOS_COIN_DECIMALS = 8

TORTUGA_STAKED_APT_COIN_TYPE = (
    "0x84d7aeef42d38a5ffc3ccef853e1b82e4958659d16a7de736a29c55fbbeb0114::staked_aptos_coin::StakedAptosCoin"
)
TORTUGA_STAKED_APT_COIN_DECIMALS = 8



from mm_crypto_utils import Proxies, random_proxy
from mm_std import Err, Ok, Result, hr, hra


def address_to_primary_name(address: str, timeout: float = 5, proxies: Proxies = None, attempts: int = 3) -> Result[str | None]:
    result: Result[str | None] = Err("not_started")
    url = f"https://www.aptosnames.com/api/mainnet/v1/primary-name/{address}"
    for _ in range(attempts):
        res = hr(url, proxy=random_proxy(proxies), timeout=timeout)
        data = res.to_dict()
        try:
            if res.code == 200 and res.json == {}:
                return Ok(None, data=data)
            return Ok(res.json["name"], data=data)
        except Exception as e:
            result = Err(e, data=data)
    return result


async def address_to_primary_name_async(
    address: str, timeout: float = 5, proxies: Proxies = None, attempts: int = 3
) -> Result[str | None]:
    result: Result[str | None] = Err("not_started")
    url = f"https://www.aptosnames.com/api/mainnet/v1/primary-name/{address}"
    for _ in range(attempts):
        res = await hra(url, proxy=random_proxy(proxies), timeout=timeout)
        data = res.to_dict()
        try:
            if res.code == 200 and res.json == {}:
                return Ok(None, data=data)
            return Ok(res.json["name"], data=data)
        except Exception as e:
            result = Err(e, data=data)
    return result


def address_to_name(address: str, timeout: float = 5, proxies: Proxies = None, attempts: int = 3) -> Result[str | None]:
    result: Result[str | None] = Err("not_started")
    url = f"https://www.aptosnames.com/api/mainnet/v1/name/{address}"
    for _ in range(attempts):
        res = hr(url, proxy=random_proxy(proxies), timeout=timeout)
        data = res.to_dict()
        try:
            if res.code == 200 and res.json == {}:
                return Ok(None, data=data)
            return Ok(res.json["name"], data=data)
        except Exception as e:
            result = Err(e, data=data)
    return result


async def address_to_name_async(
    address: str, timeout: float = 5, proxies: Proxies = None, attempts: int = 3
) -> Result[str | None]:
    result: Result[str | None] = Err("not_started")
    url = f"https://www.aptosnames.com/api/mainnet/v1/name/{address}"
    for _ in range(attempts):
        res = await hra(url, proxy=random_proxy(proxies), timeout=timeout)
        data = res.to_dict()
        try:
            if res.code == 200 and res.json == {}:
                return Ok(None, data=data)
            return Ok(res.json["name"], data=data)
        except Exception as e:
            result = Err(e, data=data)
    return result



from mm_std import DataResult, http_request


async def address_to_primary_name(address: str, timeout: float = 5, proxy: str | None = None) -> DataResult[str | None]:
    url = f"https://www.aptosnames.com/api/mainnet/v1/primary-name/{address}"
    res = await http_request(url, proxy=proxy, timeout=timeout)
    if res.is_error():
        return res.to_data_result_err()
    json_res = res.parse_json_body()
    if res.status_code == 200 and json_res == {}:
        return DataResult(ok=None, data=res.model_dump(), ok_is_none=True)
    if "name" in json_res:
        return DataResult(ok=json_res["name"], data=res.model_dump())
    return DataResult(err="unknown_response", data=res.model_dump())
